@@ -36,13 +36,12 @@ var _ = Describe("Generating external kafka server output store config block", f
 	           flush_mode interval
 	           flush_interval 1s
 	           flush_thread_count 2
-	           flush_at_shutdown true
 	           retry_type exponential_backoff
 	           retry_wait 1s
 	           retry_max_interval 60s
-	           retry_forever true
+	           retry_timeout 60m
 	           queued_chunks_limit_size "#{ENV['BUFFER_QUEUE_LIMIT'] || '32' }"
-	            total_limit_size "#{ENV['TOTAL_LIMIT_SIZE_PER_BUFFER'] ||  8589934592 }" #8G
+	           total_limit_size "#{ENV['TOTAL_LIMIT_SIZE_PER_BUFFER'] ||  8589934592 }" #8G
 	           chunk_limit_size "#{ENV['BUFFER_SIZE_LIMIT'] || '8m'}"
 	           overflow_action block
 	       </buffer>
@@ -87,9 +86,7 @@ var _ = Describe("Generating external kafka server output store config block", f
 	       brokers broker1-kafka.svc.messaging.cluster.local:9092
 	       default_topic topic
 	       use_event_time true
-	       ssl_ca_cert '/var/run/ocp-collector/secrets/some-secret/ca-bundle.crt'
-              ssl_client_cert "#{File.exist?('/var/run/ocp-collector/secrets/some-secret/tls.crt') ? '/var/run/ocp-collector/secrets/some-secret/tls.crt' : nil}"
-              ssl_client_cert_key "#{File.exist?('/var/run/ocp-collector/secrets/some-secret/tls.key') ? '/var/run/ocp-collector/secrets/some-secret/tls.key' : nil}"
+	       sasl_over_ssl false
 	       <format>
 	           @type json
 	       </format>
@@ -99,11 +96,10 @@ var _ = Describe("Generating external kafka server output store config block", f
 	           flush_mode interval
 	           flush_interval 1s
 	           flush_thread_count 2
-	           flush_at_shutdown true
 	           retry_type exponential_backoff
 	           retry_wait 1s
 	           retry_max_interval 60s
-	           retry_forever true
+	           retry_timeout 60m
 	           queued_chunks_limit_size "#{ENV['BUFFER_QUEUE_LIMIT'] || '32' }"
 	           total_limit_size "#{ENV['TOTAL_LIMIT_SIZE_PER_BUFFER'] ||  8589934592 }" #8G
 	           chunk_limit_size "#{ENV['BUFFER_SIZE_LIMIT'] || '8m'}"
@@ -147,12 +143,11 @@ var _ = Describe("Generating external kafka server output store config block", f
 	         flush_mode interval
 	           flush_interval 1s
 	           flush_thread_count 2
-	           flush_at_shutdown true
 	         retry_type exponential_backoff
 	         retry_wait 1s
 	           retry_max_interval 60s
-	           retry_forever true
-            queued_chunks_limit_size "#{ENV['BUFFER_QUEUE_LIMIT'] || '32' }"
+	           retry_timeout 60m
+	         queued_chunks_limit_size "#{ENV['BUFFER_QUEUE_LIMIT'] || '32' }"
 	           total_limit_size "#{ENV['TOTAL_LIMIT_SIZE_PER_BUFFER'] ||  8589934592 }" #8G
 	           chunk_limit_size "#{ENV['BUFFER_SIZE_LIMIT'] || '8m'}"
 	           overflow_action block
